@@ -10,6 +10,8 @@
 
 #pragma once
 
+#include "BuildSettings.h"
+
 OPENMPT_NAMESPACE_BEGIN
 
 #define VER_HELPER_STRINGIZE(x) #x
@@ -17,14 +19,11 @@ OPENMPT_NAMESPACE_BEGIN
 
 //Version definitions. The only thing that needs to be changed when changing version number.
 #define VER_MAJORMAJOR  1
-#define VER_MAJOR      27
-#define VER_MINOR      17
-#define VER_MINORMINOR 54
-
-//Version string. For example "1.17.02.28"
-#define MPT_VERSION_STR         VER_STRINGIZE(VER_MAJORMAJOR) "." VER_STRINGIZE(VER_MAJOR) "." VER_STRINGIZE(VER_MINOR) "." VER_STRINGIZE(VER_MINORMINOR)
+#define VER_MAJOR      28
+#define VER_MINOR      10
+#define VER_MINORMINOR 76
 
 //Numerical value of the version.
-#define MPT_VERSION_NUMERIC     MAKE_VERSION_NUMERIC(VER_MAJORMAJOR,VER_MAJOR,VER_MINOR,VER_MINORMINOR)
+#define MPT_VERSION_CURRENT MAKE_VERSION_NUMERIC(VER_MAJORMAJOR,VER_MAJOR,VER_MINOR,VER_MINORMINOR)
 
 OPENMPT_NAMESPACE_END
